/**
 * Originally written by Tom Haupt's group, NPAC.  
 * Modified by Marlon Pierce:  I added get and set methods for the
 * kerberos id.  I also modified the getBinaryBlocks method so that it would
 * send http header information back to the client.  The 
 * UrlConnection.getInputStream() method only returns the html for a requested
 * http site.  
 * 
 * I also am adding a getOutputStream to this so that it talks back to the 
 * server, telling it to set cookies.  I think this is the normal 
 * browser/server two-way communication, and we need this to use java's 
 * Cookie methods since these depend on request and response objects.
 * 
 * TO DO:  
 * 1. Also need to modify the getBlocks (which returns strings instead of
 * binary) to return the header stuff.
 *
 * 2. Should be able to return binaries in streams using one of the java
 * IO classes instead of the getBinaryBlocks() method.
 *
 * 3. Currently, I am just trying to set a particular cookie on the 
 * server side, so I don't communicate directly with the browser.  However,
 * in the future, this should be changed so that the entire browser request
 * gets transfered from MyCharon to CharonServerImp.java.
 */

package WebFlow.Charon;

import WebFlow.*;
import WebFlow.event.*;
import WebFlow.Charon.CharonServerPackage.*;
import org.omg.CORBA.*;
import java.util.*;
import java.net.*;
import java.io.*; 

public class CharonServerImp extends WebFlow.BeanContextChildSupport 
    implements CharonServerOperations {
    
    String msg_;
    org.omg.CORBA.Object peer;
    BufferedInputStream bif;
    BufferedOutputStream bos;
    int blksize; 
    String myID,myCookie;
    int testint;
    boolean headerflag;
    URLConnection conn;
    
    public CharonServerImp(org.omg.CORBA.Object peer, String msg)
	throws WebFlow.NullPointerException{
	
	super(peer);
	this.peer = peer;
	String msg_ = msg;
	blksize=1024;	
    }

    public void test(){
	System.out.println("Charon Server is up!");
    }  
    
    public boolean connect(String UrlString) {
	boolean status=true; 
	headerflag=true;
	try {
	    URL url = new URL(UrlString);
	    //      InputStream is = url.openStream();
	    //I added the stuff below--Marlon.
	    conn=url.openConnection();
	    conn.setDoOutput(true);
	    conn.setDoInput(true);
	    conn.setDefaultUseCaches(false);
	    conn.setUseCaches(false);
	    conn.setRequestProperty("Content-Type",
                                    "application/x-www-form-urlencoded");
	    conn.setRequestProperty("Cookie","kerID="+myID);
	    System.out.println("Kerid is set: "+myID);
	    if(myCookie!=null){
		conn.setRequestProperty("Cookie","JESSIONID="+myCookie
					+"; kerID="+myID );
		System.out.println("JSESSIONID is set: "+myCookie);
	    }

	    conn.connect();
	    //	    System.out.println("Test header: "+conn.getHeaderField("Set-Cookie"));
	    InputStream is=conn.getInputStream();
	    bif = new BufferedInputStream(is);

	    //Talk back to the server. --Marlon
//  	    OutputStream out=conn.getOutputStream();
//  	    bos=new BufferedOutputStream(out);
	    
	} catch (Exception e) {
	    System.out.println("cannot connect to "+UrlString);
	    System.out.println(e.toString());
	    status=false;
	}
	return status;
    } 
    
    public void closeConnection() {
	System.out.println("close connection"); 
	try {
	    if(bif != null) bif.close(); 
	    else System.out.println("bif is already closed");
	} catch (Exception e) {
	    System.out.println("cannot close input stream");
	    System.out.println(e.toString());
	}
	bif=null; 
    } 
    
    public void setBlockSize(int size) {
	blksize=size;
    }
    
    public String getBlock() {
	String response="";    
	byte[] a = null; 
	byte[] b = new byte[blksize];
	try {
	    if(bif==null) System.out.println("bif is null");
	    int icount = bif.read(b);
	    System.out.println("icount="+icount);
	    if(icount == -1) {
		response="EOF";
	    }
	    else { 
		if(icount == blksize) a=b;  //buffer filled
		else {
		    a = new byte[icount];
		    for(int i =0; i < icount; i++) a[i]=b[i];
		}
		response=new String(a); 
	    }
	}
	catch(Exception e) {
	    System.out.println("IO Exception in getBlock");
	    System.out.println(e);
	}
	

	/* you may want to modify a
	   response=checkString(response,TargetHost);
	*/
	if(response==null) response="null";
	return response;
    }
    
    public byte[] getBinaryBlock() {

	
	//I have modified this to first send the header info to 
	//the client (in binary) and then send the contents of the
	//URL.
	byte[] a = null; 
	byte[] b = new byte[blksize];
	//Return header information first
	//System.out.println("In binary block");
	if(headerflag) {
	    String holder="HTTP/1.0 200 OK \n";
	    String key;
	    int n=1;
	    while((key=conn.getHeaderFieldKey(n)) !=null){
		holder+=key+": "+
		    conn.getHeaderField(key) +"\n";
		n++;
	    }
	    System.out.println(holder);
	    holder+="\n";
	    a=holder.getBytes();
	    headerflag=false;
	}
	else {
	    try {
		if(bif==null) System.out.println("bif is null");
		int icount = bif.read(b);
		//System.out.println("icount="+icount);
		if(icount == -1) {
		    a = new byte[0];
		}
		else { 
		    if(icount == blksize) a=b;  //buffer filled
		    else {
			a = new byte[icount];
			for(int i =0; i < icount; i++) a[i]=b[i];
		    }
		}
	    }
	    catch(Exception e) {
		System.out.println("IO Exception in getBlock");
		System.out.println(e);
	    }
	    
	}
	return a;
    }
    
    private  String checkString(String string, String TargetServer) {
	/* replaces relative references to absolute ones
	   for example:
	   <a href="/page.html"> to <a href="http://TargetServer/page.html">
	*/
	
	String newString = removeAbsolute(string,"HREF",TargetServer); 
	newString = removeAbsolute(newString,"href",TargetServer); 
	newString = removeAbsolute(newString,"SRC",TargetServer); 
	newString = removeAbsolute(newString,"src",TargetServer); 
	return newString;
    }
    
    private String removeAbsolute(String string, String Tag, 
				  String TargetServer){
	
	String newstring = null;
	String temp,temp1,temp2; 
	
	temp1="";      
	int idx=string.indexOf(Tag);
	if(idx<0) return string;
	while(idx>-1) {
	    newstring=string.substring(0,idx);
	    temp=string.substring(idx);
	    /* now temp starts with Tag: move Tag to newstring */
	    int idx2 = temp.indexOf("=")+1;
	    newstring += temp.substring(0,idx2);
	    temp = temp.substring(idx2);
	    /* now temp starts after = */  
	    if(temp.startsWith("\"/") || 
	       temp.startsWith(" \"/") ||
	       temp.startsWith("/") || 
	       temp.startsWith(" /")) {
		temp = "http://"+TargetServer+temp;
	    }
	    newstring += temp;  
	    temp1 += newstring;
	    string = temp;
	    string=string.substring(Tag.length());
	    idx=string.indexOf(Tag);
	}
	return temp1;
    }
    
    public synchronized void waitfordata() {
	try {
	    System.out.println("wait"); 
	    wait(100);
	} catch(Exception e) {};
    } 
    
    public void setKerberosID(String astring) {
	this.myID=astring;
    }
    
    
    public String getKerberosID() {
	//Returns the id as a string.
	return this.myID;
    }
 
  public void setSessionCookie(String cookieString) {
    this.myCookie=cookieString;
  }

  public String getSessionCookie(){
    return this.myCookie;
  }

}
